use std::sync::Arc;

use octopush_core::{
    builder::{builder_capabilities::BuilderCapabilities, DynBuilder},
    executor::{default_executor::DefaultExecutor, executor::DynExecutor},
    git::{
        git::LocalGitProvider,
        gitea::{provider::DefaultGiteaProvider, DynGiteaProvider},
        DynGitProvider,
    },
    schema::parser::{DefaultSchemaParser, DynSchemaParser},
    storage::{local::LocalStorageEngine, DynStorageEngine},
};

pub struct ServiceRegister {
    pub storage_engine: DynStorageEngine,
    pub git_provider: DynGitProvider,
    pub schema_parser: DynSchemaParser,
    pub builder: DynBuilder,
    pub executor: DynExecutor,
    pub gitea_provider: DynGiteaProvider,
}

impl ServiceRegister {
    pub fn new() -> Self {
        let storage_engine = Arc::new(LocalStorageEngine::new("/tmp/octopush".into()));
        let git_provider = Arc::new(LocalGitProvider::new(storage_engine.clone()));
        let schema_parser = Arc::new(DefaultSchemaParser::new());
        let builder = Arc::new(BuilderCapabilities::new());
        let executor = Arc::new(DefaultExecutor::new(builder.clone()));
        let gitea_provider = Arc::new(DefaultGiteaProvider::new(git_provider.clone()));

        Self {
            storage_engine,
            git_provider,
            schema_parser,
            builder,
            executor,
            gitea_provider,
        }
    }

    pub async fn cleanup(self) -> eyre::Result<()> {
        self.storage_engine.cleanup().await
    }
}
